/* Provider schema block -- version 1 */

CREATE TABLE mysql_Membership(`PKID` varchar(36) NOT NULL,
Username varchar(255) NOT NULL,
ApplicationName varchar(255) NOT NULL,
Email varchar(128) NOT NULL,
Comment varchar(255) default NULL,
Password varchar(128) NOT NULL,
PasswordQuestion varchar(255) default NULL,
PasswordAnswer varchar(255) default NULL,
IsApproved tinyint(1) default NULL,
LastActivityDate datetime default NULL,
LastLoginDate datetime default NULL,
LastPasswordChangedDate datetime default NULL,
CreationDate datetime default NULL,
IsOnline tinyint(1) default NULL,
IsLockedOut tinyint(1) default NULL,
LastLockedOutDate datetime default NULL,
FailedPasswordAttemptCount int(10) unsigned default NULL,
FailedPasswordAttemptWindowStart datetime default NULL,
FailedPasswordAnswerAttemptCount int(10) unsigned default NULL,
FailedPasswordAnswerAttemptWindowStart datetime default NULL,
PRIMARY KEY (`PKID`)) ENGINE=MyISAM DEFAULT CHARSET=latin1 COMMENT='1';

CREATE TABLE mysql_UsersInRoles(`Username` varchar(255) NOT NULL,
`Rolename` varchar(255) NOT NULL, `ApplicationName` varchar(255) NOT NULL,
KEY `Username` (`Username`,`Rolename`,`ApplicationName`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 ROW_FORMAT=DYNAMIC;

CREATE TABLE mysql_Roles(`Rolename` varchar(255) NOT NULL,
`ApplicationName` varchar(255) NOT NULL,
KEY `Rolename` (`Rolename`,`ApplicationName`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 ROW_FORMAT=DYNAMIC;

/* Provider schema block -- version 2 */
ALTER TABLE mysql_Membership
ADD PasswordKey char(32) AFTER Password,
ADD PasswordFormat tinyint AFTER PasswordKey,
CHANGE email email VARCHAR(128), COMMENT='2';

/* Provider schema block -- version 3 */

/* create our application and user tables */
create table my_aspnet_Applications(id INT PRIMARY KEY AUTO_INCREMENT, name VARCHAR(256), description VARCHAR(256));
create table my_aspnet_Users(id INT PRIMARY KEY AUTO_INCREMENT, applicationId INT, name VARCHAR(256), isAnonymous TINYINT(1), lastActivityDate DATETIME);
create table my_aspnet_Profiles(userId INT PRIMARY KEY, valueindex longtext, stringdata longtext, binarydata longblob, lastUpdatedDate timestamp);
create table my_aspnet_SchemaVersion(version INT);

insert into my_aspnet_SchemaVersion VALUES (3);

/* now we need to migrate all applications into our apps table */
insert into my_aspnet_Applications (name) select ApplicationName from mysql_Membership UNION select ApplicationName from mysql_UsersInRoles;

/* now we make our changes to the existing tables */
alter table mysql_Membership
rename to my_aspnet_Membership,
drop primary key,
drop column pkid,
drop column isonline,
add column userId INT FIRST,
add column applicationId INT AFTER userId;

alter table mysql_Roles
rename to my_aspnet_Roles,
drop key rolename,
add column id INT PRIMARY KEY AUTO_INCREMENT FIRST,
add column applicationId INT AFTER id;

alter table mysql_UsersInRoles
drop key username,
rename to my_aspnet_UsersInRoles,
add column userId INT FIRST,
add column roleId INT AFTER userId,
add column applicationId INT AFTER roleId;

/* these next lines set the application Id on our tables appropriately */
update my_aspnet_Membership m, my_aspnet_Applications a set m.applicationId = a.id where a.name=m.ApplicationName;
update my_aspnet_Roles r, my_aspnet_Applications a set r.applicationId = a.id where a.name=r.ApplicationName;
update my_aspnet_UsersInRoles u, my_aspnet_Applications a set u.applicationId = a.id where a.name=u.ApplicationName;

/* now merge our usernames into our users table */
insert into my_aspnet_Users (applicationId, name)
select applicationId, username from my_aspnet_Membership
UNION select applicationId, username from my_aspnet_UsersInRoles;

/* now set the user ids in our tables accordingly */
update my_aspnet_Membership m, my_aspnet_Users u set m.userId = u.id where u.name=m.UserName AND u.applicationId=m.ApplicationId;
update my_aspnet_UsersInRoles r, my_aspnet_Users u set r.userId = u.id where u.name=r.UserName AND u.applicationId=r.applicationId;

/* now update the isanonymous and last activity date fields for the users */
update my_aspnet_Users u, my_aspnet_Membership m
set u.isAnonymous=0, u.lastActivityDate=m.lastActivityDate
where u.name = m.username;

/* make final changes to our tables */
alter table my_aspnet_Membership
drop column username,
drop column applicationName,
drop column applicationId,
add primary key (userId);

/* next we set our role id values appropriately */
update my_aspnet_UsersInRoles u, my_aspnet_Roles r set u.roleId = r.id where u.rolename = r.rolename and r.applicationId=u.applicationId;

/* now we make the final changes to our roles tables */
alter table my_aspnet_Roles
drop column applicationName,
change column rolename name VARCHAR(255) NOT NULL;

alter table my_aspnet_UsersInRoles
drop column applicationName,
drop column applicationId,
drop column username,
drop column rolename,
add primary key (userId, roleId);

/* update membership comment to 3 - representing this schema version */
ALTER TABLE my_aspnet_Membership
COMMENT='3';
